/* File: CSplitSys.h
 * Version 1.0 (AUG 25, 2010)
 * --------------------------------------------------------------------------
 * Objective: Define the class CSplitSys
 *
 *      Private members:
 **             m_iNumber: the number taxa in the split system                
 **               m_gName: store the ordered taxa names
 **            m_gQuartet: store the quartets together with their weights 
 **            m_gQuintet: store the 2|3 splits together with their weights 
 **             m_gSplits: store the full splits together with their weights    
 *     
 *      Public member functions:
 **            ProcessMin: generate full splits and output to file by the minimum method
 *
 *	Written by Jialiang Yang
 * --------------------------------------------------------------------------
 */


#ifndef _CSPLIT_SYS_H
#define _CSPLIT_SYS_H

// Body of CSplitSys.h
#include <string>
#include "CTriplet.h"
#include "CQuartet.h"
#include "CQuintet.h"
#include "CSplit.h"

using namespace std;

class CSplitSys
{
public:
    // Default constructor
	CSplitSys(): m_iNumber(), m_gName(), m_gTriplet(), m_gQuartet(), m_gQuintet(), m_gSplit(){} 
	 
    // Constructor with parameters
    CSplitSys::CSplitSys(unsigned int iNumber, const vector<string>& gName, const vector<CTriplet>& gTriplet, 
	const vector<CQuartet>& gQuartet, const vector<CQuintet>& gQuintet, const vector<CSplit>& gSplit): m_iNumber(iNumber), 
	m_gName(gName), m_gTriplet(gTriplet), m_gQuartet(gQuartet), m_gQuintet(gQuintet), m_gSplit(gSplit){}
    
    // Copy constructor                    
	CSplitSys(const CSplitSys& rhs): m_iNumber(rhs.m_iNumber), m_gName(rhs.m_gName), m_gTriplet(rhs.m_gTriplet), 
	m_gQuartet(rhs.m_gQuartet), m_gQuintet(rhs.m_gQuintet), m_gSplit(rhs.m_gSplit){}         
                                    
	CSplitSys& operator=(const CSplitSys& );                                     // Operator constructor

    // Set and return member variables
    unsigned int GetNumber() const { return m_iNumber;}                          // Return taxa number
    vector<string> GetName() const { return m_gName;}                            // Return taxa names.
	vector<CTriplet> GetTriplet() const { return m_gTriplet;}                    // Return all Triplets
    vector<CQuartet> GetQuartet() const { return m_gQuartet;}                    // Return all quartets 
    vector<CQuintet> GetQuintet() const { return m_gQuintet;}                    // Return all quintets    
	vector<CSplit> GetSplit() const { return m_gSplit;}                          // Return full splits     
    void SetNumber(unsigned int iNumber){ m_iNumber = iNumber;}                  // Set taxa number to be iNumber
    void SetName(const vector<string>& gName){ m_gName = gName; }                // Set taxa names to be vector gName
	void SetTriplet(const vector<CTriplet>& gTriplet) {m_gTriplet = gTriplet;}   // Set Triplets to be vector gTriplet
	void SetQuartet(const vector<CQuartet>& gQuartet) {m_gQuartet = gQuartet;}   // Set Quartets to be vector gQuartet
    
    // Ordering, shrinking and expand split system 
    void OrderQuartets();                                                        // Reoder the quatets in the split system according to their index 
	void OrderTriplets();                                                        // Reoder the triplets in the split system according to their index 
	void ShrinkSplits(vector<CSplit>&);                                          // Delete splits that have weight 0
    void ExpandAlignmentMin();                                                   // Generating the non-trivial full splits
	void GenerateTrivialSplit();                                                 // Generating trivial full splits
      
    // Import data from files
    void ReadAlignment(const string&);                                           // Import data from multiple alignment file: fasta format
    void ReadWeight(const string&);                                              // Import data from weight files as formatted in the manual
   
     // Weight functions
     double MinQuintet(unsigned int gLeft[2], unsigned int gRight[3]);           // calcualte weights of split types 2|3 using minimum weight function
	 double SecondQuintet(unsigned int gLeft[2], unsigned int gRight[3]);        // calcualte weights of split types 2|3 using second weight function
     double MinWei24(unsigned int gLeft[2], unsigned int gRight[4]);             // Calculate split of type 2|4 by min method
     double MinWei2n(const vector<unsigned int>&, const vector<unsigned int>&);  // Calculate split of type 2|n by min method
     double MinWei33(unsigned int gLeft[3], unsigned int gRight[3]);             // Calculate split of type 3|3 by min method
     double MinWeimn(const vector<unsigned int>&, const vector<unsigned int>&);  // Calculate split of type m|n by min method
     void GenerateQuintet();                                                     // Generate and order all 2|3 splits using min method
	 void GenerateQuintetSecond();                                               // Generate and order all 2|3 splits using second method
     
	 // Output
	 void Output(string&);                                                       // Output the full splits into sOutputFile.nex
	 void ProcessAlnMin(string&, string&);                                       // generate full splits and output to file by the minimum method from alignment file
	 void ProcessWeiMin(string&, string&);                                       // generate full splits and output to file by the minimum method from weight file
	 void ProcessAlnSecond(string&, string&);                                    // generate full splits and output to file by the second method from alignment file
	 void ProcessWeiSecond(string&, string&);                                    // generate full splits and output to file by the second method from weight file
                                                                        
private:
    unsigned int m_iNumber;                                                      // Number of taxa in the system
    vector<string> m_gName;                                                      // Names of ordered taxa
	vector<CTriplet> m_gTriplet;                                                 // Ordered triplet i.e. 1|2 splits
    vector<CQuartet> m_gQuartet;                                                 // Ordered quartets 2|2 splits
    vector<CQuintet> m_gQuintet;                                                 // Ordered quintets 2|3 splits
	vector<CSplit> m_gSplit;                                                     // Ordered Full splits
};

#endif
// EOF


