/* File: ReadAlignment.cpp
 * Version 1.0 (AUG 25, 2010)
 * -----------------------------------------------------------------------------------------
 * Objective: import the triplets and quartets information from multiple alignment file
 *
 *	Written by Jialiang Yang
 * -----------------------------------------------------------------------------------------
 */
 
#include <fstream>
#include <sstream>
#include <iostream>
#include <vector>

#include "CSplitSys.h"
#include "auxilary.h"

using namespace std;

// Import data from multiple alignment file: fasta format
void CSplitSys::ReadAlignment(const string& sInputFile)
{
     // Open file filename for reading
     ifstream fin(sInputFile.c_str());                                 
               
     if(!fin ) 
     {   
         cerr << "Error to  open " << sInputFile.c_str() << " for reading!" << endl;  
         system("pause"); 
         exit(-1);  
     }
     else
     {    
        string sLine;                   // sLine stores the strings in a line
		vector<string> gSeq;            // stores the sequences of the taxa set
				
        string sSeq = "";
                
	  // Read the file line by line 
         while(getline(fin, sLine) )                                         
         {   
             if(sLine == "") continue;       // If it is a blank line, go back and read the next line
             else
		     {
               if(sLine[0] == '>')           // Indicate that the sequence of a new species comes 
                 { 
                      gSeq.push_back(sSeq);  // push back the sequence of the previous species to gSeq
                 
					  m_gName.push_back(sLine.substr(1, sLine.size()-1));
						  
                      sSeq = "";
                 }    
                 else
                 {
				     sLine = trim(sLine);      // remove the possible spaces at the begining and end of a line
                     sSeq += sLine;
                 }                
             }
         }
		
        gSeq.push_back(sSeq);                  // push the last sequence
        m_iNumber = m_gName.size();
		
		
	 // Calculate triplet weights from the multiple alignment 

		// Set all the 3*(n,3)  triplets to be empty triplets;
 	    CTriplet tEmpty;
        for(unsigned long long n = 0; n != 3*Binom(m_iNumber,3); ++n)
             m_gTriplet.push_back(tEmpty);
        
        unsigned int gRight[2];    
        
		string::size_type i, j, k , l; 
		
		for(i = 1; i != m_iNumber-1; ++i)
		   for( j = i+1; j != m_iNumber; ++j)
			   for( k = j+1; k != m_iNumber +1; ++k)
		{
		      // initialize 3 triplets 
			  
			  // Triplet 1: i|kj, e.g. 1|32
			  gRight[0] = k;
			  gRight[1] = j;
			  
		      double dWei = (double) CalTriplet(gSeq[i], gSeq[k], gSeq[j]);
		      CTriplet tOne(i, gRight, dWei);
		      
		      // push triplet 1 to the corresponding position of  m_gTriplet
			  m_gTriplet[tOne.GetIndex()] = tOne;
			  
			  // push triplet 2: j|ki to m_gTriplet 2|31
			  dWei = (double) CalTriplet(gSeq[j], gSeq[k], gSeq[i]);
              gRight[1] = i;
              
		      CTriplet tTwo(j, gRight, dWei);
			  m_gTriplet[tTwo.GetIndex()] = tTwo;
			  
			  // push triplet 3: k|ji to m_gTriplet 3|21
			  dWei = (double) CalTriplet(gSeq[k], gSeq[j], gSeq[i]);
              gRight[0] = j;
              
		      CTriplet tThree(k, gRight, dWei);
			  m_gTriplet[tThree.GetIndex()] = tThree;
		}
		
	// Calculate quartet weights from the multiple alignment 	
        
	    // specify all 3(n,4) quartets to be empty quartets
       	CQuartet qEmpty;
        for(unsigned long long n = 0; n != 3*Binom(m_iNumber,4); ++n)
             m_gQuartet.push_back(qEmpty);
       
       unsigned int gLeft[2];      
       
       for(i = 1; i != m_iNumber-2; ++i)
		  for( j = i+1;j != m_iNumber-1; ++j)
			 for( k = j+1; k != m_iNumber; ++k)
                 for( l = k+1; l != m_iNumber+1; ++l)
                 {
                         // Spcify quartet 1: li|kj and its weight w(li|kj)
                         gLeft[0] = l;
                         gLeft[1] = i;
                         gRight[0] = k;
                         gRight[1] = j;
                         double dWei = (double) CalQuartet(gSeq[l], gSeq[i], gSeq[k], gSeq[j]);
                         CQuartet qOne(gLeft, gRight, dWei);
                         m_gQuartet[qOne.GetIndex()] = qOne;
                         
                         // Spcify quartet 2: lj|ki and its weight w(lj|ki)
                         swap(gLeft[1], gRight[1]);
                         dWei = (double)CalQuartet(gSeq[l], gSeq[j], gSeq[k], gSeq[i]);
                         CQuartet qTwo(gLeft, gRight, dWei);
                         m_gQuartet[qTwo.GetIndex()] = qTwo;

                         // Spcify quartet 3: lk|ji and its weight w(lk|ji)
                         swap(gLeft[1], gRight[0]);
                         dWei=(double)CalQuartet(gSeq[l], gSeq[k], gSeq[j], gSeq[i]);
                         CQuartet qThree(gLeft, gRight, dWei);
                         m_gQuartet[qThree.GetIndex()] = qThree;
                 } 
		}
		
    // Output the quartets into a file
    ofstream fout;
     
     size_t pos = sInputFile.find('.');
     string sQweightFile = sInputFile.substr(6, pos-6);
     string sOutputFile  = "QweightFile//"+sQweightFile + ".gWeights";
	 
     fout.open( sOutputFile.c_str(),   ios_base::out   |   ios_base::trunc);  
		    
     if(!fout) 
     {   
        cerr << "Error opening " << sOutputFile << " for output!" << endl;  
        system("pause"); 
        exit(-1);  
     }
     else
     {
         fout << "taxanumber:" << m_iNumber << ";" << endl;
         
         unsigned int iNum = 1;
         for(vector<string>::iterator it = m_gName.begin(); it != m_gName.end(); ++it)
         {
            fout << "taxon: " << "\t" << iNum << "\t" << "  name: " << "\t" << *it << ";" <<endl;
            iNum++;
         }
         
         for(vector<CQuartet>::iterator it = m_gQuartet.begin(); it != m_gQuartet.end(); ++it)
         {
             const unsigned int *gLeft = (*it).GetLblk();
             const unsigned int *gRight = (*it).GetRblk();
             
             fout << "Quartet: " << gLeft[0]  << "\t" << gLeft[1]  << "\t" << gRight[0]
              << "\t" << gRight[1] << "\t" <<(*it).GetWeight() <<";" << endl;
         }
     }
}
  
// EOF
